#include <stdlib.h>
#include "miniglut.h"
#include "demo.h"
#include "opt.h"

static void display(void);
static void idle(void);
static void keydown(unsigned char key, int x, int y);
static void keyup(unsigned char key, int x, int y);
static void mouse(int bn, int st, int x, int y);

static long time_start;

int main(int argc, char **argv)
{
	unsigned int glut_flags = GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH;

	glutInit(&argc, argv);

	read_cfg("demo.cfg");
	if(parse_args(argc, argv) == -1) {
		return 1;
	}
	if(opt.srgb) glut_flags |= GLUT_SRGB;
	if(opt.msaa) glut_flags |= GLUT_MULTISAMPLE;

	glutInitWindowSize(opt.width, opt.height);
	glutInitDisplayMode(glut_flags);
	glutCreateWindow("Prior Art / Mindlapse");

	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(demo_reshape);
	glutKeyboardFunc(keydown);
	glutKeyboardUpFunc(keyup);
	glutMouseFunc(mouse);
	glutMotionFunc(demo_mmotion);
	glutSpaceballMotionFunc(demo_sball_motion);
	glutSpaceballRotateFunc(demo_sball_rotate);
	glutSpaceballButtonFunc(demo_sball_button);

	if(opt.fullscr) {
		glutFullScreen();
	}

	if(demo_init() == -1) {
		return 1;
	}
	atexit(demo_cleanup);

	time_start = glutGet(GLUT_ELAPSED_TIME);

	glutMainLoop();
	return 0;
}

void demo_quit(void)
{
	glutExit();
}

static void display(void)
{
	time_msec = glutGet(GLUT_ELAPSED_TIME) - time_start;

	demo_display();

	glutSwapBuffers();
}

static void idle(void)
{
	glutPostRedisplay();
}

#define KEYCOMBO_FS \
	((key == '\r' || key == '\n') && (glutGetModifiers() & GLUT_ACTIVE_ALT))


static void keydown(unsigned char key, int x, int y)
{
	static int fullscr = -1, prev_x, prev_y;

	if(KEYCOMBO_FS) {
		if(fullscr == -1) fullscr = opt.fullscr;
		fullscr ^= 1;
		if(fullscr) {
			prev_x = glutGet(GLUT_WINDOW_X);
			prev_y = glutGet(GLUT_WINDOW_Y);
			glutFullScreen();
		} else {
			glutPositionWindow(prev_x, prev_y);
		}
	}

	demo_keyboard(key, 1);
}

static void keyup(unsigned char key, int x, int y)
{
	demo_keyboard(key, 0);
}

static void mouse(int bn, int st, int x, int y)
{
	demo_mbutton(bn - GLUT_LEFT_BUTTON, st == GLUT_DOWN, x, y);
}
